/*
 * Copyright (c) 1996, 2003 VIA Networking Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    This product includes software developed by VIA Networking Technologies, Inc.
 * 4. Neither the name of the Company nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY VIA NETWORKING TECHNOLOGIES, INC. AND 
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL 
 * VIA NETWORKING TECHNOLOGIES, INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * File: if_getb.h
 *
 * Purpose: FreeBSD driver header file
 *
 * Author: Ryan Fu
 *
 * Date: Jul 22, 2003
 *
 *
 */

#ifndef __IF_GETB_H__
#define __IF_GETB_H__

#include <sys/param.h>

#if __FreeBSD_version < 400000
#include "bpfilter.h"
#endif /*__FreeBSD_version*/

#include <sys/systm.h>
#include <sys/sockio.h>
#include <sys/mbuf.h>
#include <sys/malloc.h>
#include <sys/kernel.h>
#include <sys/socket.h>
#include <sys/protosw.h>

#include <net/if.h>
#include <net/if_arp.h>
#include <net/ethernet.h>
#include <net/if_dl.h>
#include <net/if_media.h>

#if (__FreeBSD_version < 400000 && NBPFILTER > 0) || __FreeBSD_version >= 400000
#include <net/bpf.h>
#endif /*__FreeBSD_version*/

#if __FreeBSD_version >=400000 && __FreeBSD_version <410000
#include "opt_bdg.h"
#ifdef BRIDGE
#include <net/bridge.h>
#endif /* BRIDGE */
#endif /*__FreeBSD_version*/

#include <vm/vm.h>
#include <vm/pmap.h>
#include <machine/clock.h>
#include <machine/bus_pio.h>
#include <machine/bus_memio.h>
#include <machine/bus.h>

#if __FreeBSD_version >= 400000
#include <machine/resource.h>
#include <sys/bus.h>
#include <sys/rman.h>
#endif /*__FreeBSD_version*/

#include <pci/pcireg.h>
#include <pci/pcivar.h>

#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>

#include <dev/getb/if_getb_kc.h>
#include <dev/getb/if_getb_cfg.h>
#include <dev/getb/if_getb_desc.h>
#include <dev/getb/if_getb_mac.h>
#include <dev/getb/if_getb_mii.h>
#include <dev/getb/if_getb_dbg.h>


#define GETB_TX_TIMEOUT             5

#define GETB_WOL_MAGIC              0x00000000UL
#define GETB_WOL_PHY                0x00000001UL
#define GETB_WOL_ARP                0x00000002UL
#define GETB_WOL_UCAST              0x00000004UL
#define GETB_WOL_BCAST              0x00000010UL
#define GETB_WOL_MCAST              0x00000020UL
#define GETB_WOL_MAGIC_SEC          0x00000040UL

/*flags for options*/
#define GETB_FLAGS_TAGGING          0x00000001UL
#define GETB_FLAGS_TX_CSUM          0x00000002UL
#define GETB_FLAGS_RX_CSUM          0x00000004UL
#define GETB_FLAGS_VAL_PKT_LEN      0x00000008UL
/*flags for driver status*/
#define GETB_FLAGS_OPENED           0x00010000UL
#define GETB_FLAGS_VMNS_CONNECTED   0x00020000UL
#define GETB_FLAGS_VMNS_COMMITTED   0x00040000UL
#define GETB_FLAGS_WOL_ENABLED      0x00080000UL

/*flags for MII status*/
#define GETB_LINK_FAIL              0x00000001UL
#define GETB_SPEED_10               0x00000002UL
#define GETB_SPEED_100              0x00000004UL
#define GETB_SPEED_1000             0x00000008UL
#define GETB_DUPLEX_FULL            0x00000010UL
#define GETB_AUTONEG_ENABLE         0x00000020UL
#define GETB_FORCED_BY_EEPROM       0x00000040UL

/*for getb_set_media_mode*/
#define GETB_LINK_CHANGE            0x00000001UL

typedef enum __get_speed_opt {
    SPD_DPX_AUTO=0,
    SPD_DPX_100_HALF=1,
    SPD_DPX_100_FULL=2,
    SPD_DPX_10_HALF=3,
    SPD_DPX_10_FULL=4
} SPD_DPX_OPT, *PSPD_DPX_OPT;

typedef enum __get_init_type {
    GETB_INIT_COLD=0,
    GETB_INIT_RESET,
    GETB_INIT_WOL
} GETB_INIT_TYPE, *PGETB_INIT_TYPE;

typedef enum __get_flow_cntl_type {
    FLOW_CNTL_DEFAULT=1,
    FLOW_CNTL_DISABLE,
    FLOW_CNTL_ENABLE    
} GETB_FLOW_CNTL_TYPE, *PGETB_FLOW_CNTL_TYPE;

typedef struct __get_opt {
    int         nRxDescs;       /*Number of RX descriptors*/
    int         nTxDescs;       /*Number of TX descriptors*/
    SPD_DPX_OPT spd_dpx;        /*Media link mode*/
    int         vid;            /*vlan id*/
    int         DMA_length;     /*DMA length*/
    int         rx_thresh;      /*RX_THRESH*/
    int         flow_cntl;      /*flow control*/
    int         wol_opts;       /*Wake on lan options*/
    int         int_works;
    U32         flags;
} OPTIONS, *POPTIONS;

#define GETB_MEMIO_RES_ID       0x14


#if __FreeBSD_version >= 400000

typedef struct __get_info {
    struct  arpcom              arpcom;
    struct  device*             dev;
    int                         unit;
    struct resource*            mem_res;

    struct __get_info*          next;
    struct __get_info*          prev;

    bus_space_tag_t             mem_tag;
    bus_space_handle_t          mem_hdl;
    struct resource*            int_res;
    void*                       int_hdl_tag;

    struct ifmedia              media;
    struct callout_handle       timer_handle;

/*  struct net_device_stats     stats;  */

    void*                       pool;
    int                         pool_size;
    void*                       rd_pool_dma;
    void*                       td_pool_dma[TX_QUEUE_NO];

    CHIP_TYPE                   chip_id;

    PMAC_REGS                   pMacRegs;
    U32                         memaddr;
    U32                         ioaddr;
    U32                         io_size;

    U8                          byRevId;
    U16                         SubSystemID;
    U16                         SubVendorID;

#define AVAIL_TD(p,q)   ((p)->sOpts.nTxDescs-((p)->iTDUsed[(q)]))

    int                         nTxQueues;

    volatile int                iTDUsed[TX_QUEUE_NO];
    int                         aiCurrTDIdx[TX_QUEUE_NO];
    int                         aiTailTDIdx[TX_QUEUE_NO];
    PTX_DESC                    apTDRings[TX_QUEUE_NO];
    PGETB_TD_INFO               apTDInfos[TX_QUEUE_NO];

    int                         iCurrRDIdx;
    int                         iRDUsed;
    int                         iTailRDIdx;
    PRX_DESC                    aRDRing;
    PGETB_RD_INFO               aRDInfo;

#define GET_RD_BY_IDX(pInfo, idx)   (pInfo->aRDRing[idx])

    U32                         adwHWMIBCounters[MAX_HW_MIB_COUNTER];
    OPTIONS                     sOpts;

    U32                         IntMask;

    U32                         flags;

    int                         rx_buf_sz;
    U32                         mii_status;
    int                         multicast_limit;
    U32                         dwPHYId;

    U8                          abyVCAMMask[(VCAM_SIZE/8)];
    U8                          abyMCAMMask[(MCAM_SIZE/8)];

#ifdef CONFIG_PROC_FS
    PGETB_PROC_ENTRY            pProcDir;
#endif

    U32                         ticks;
    U32                         rx_bytes;

} GETB_INFO, *PGETB_INFO;

#elif __FreeBSD_version < 400000

typedef struct __get_info {
    struct  arpcom              arpcom;
    struct  ifmedia             media;        /* media info */

    /*struct  device*             dev;*/
    int                         unit;

    struct __get_info*          next;
    struct __get_info*          prev;

    bus_space_tag_t             mem_tag;
    bus_space_handle_t          mem_hdl;

    /*struct resource*            mem_res;*/
    /*struct resource*            int_res;*/
    /*void*                       int_hdl_tag;*/
    /*struct callout_handle       timer_handle;*/

    /*struct net_device_stats     stats;  */
    caddr_t                     rawpool[TX_QUEUE_NO+1];
    void*                       pool[TX_QUEUE_NO+1];
    int                         pool_size;
    void*                       rd_pool_dma;
    void*                       td_pool_dma[TX_QUEUE_NO];


    CHIP_TYPE                   chip_id;

    PMAC_REGS                   pMacRegs;
    U32                         memaddr;
    U32                         ioaddr;
    U32                         io_size;

    U8                          byRevId;
    /*U16                         SubSystemID;
      U16                         SubVendorID;
    */
#define AVAIL_TD(p,q)   ((p)->sOpts.nTxDescs-((p)->iTDUsed[(q)]))

    int                         nTxQueues;

    volatile int                iTDUsed[TX_QUEUE_NO];
    int                         aiCurrTDIdx[TX_QUEUE_NO];
    int                         aiTailTDIdx[TX_QUEUE_NO];
    PTX_DESC                    apTDRings[TX_QUEUE_NO];
    PGETB_TD_INFO               apTDInfos[TX_QUEUE_NO];

    int                         iCurrRDIdx;
    int                         iRDUsed;
    int                         iTailRDIdx;
    PRX_DESC                    aRDRing;
    PGETB_RD_INFO               aRDInfo;

#define GET_RD_BY_IDX(pInfo, idx)   (pInfo->aRDRing[idx])

    U32                         adwHWMIBCounters[MAX_HW_MIB_COUNTER];
    OPTIONS                     sOpts;

    U32                         IntMask;

    U32                         flags;

    int                         rx_buf_sz;
    U32                         mii_status;
    int                         multicast_limit;
    U32                         dwPHYId;

    U8                          abyVCAMMask[(VCAM_SIZE/8)];
    U8                          abyMCAMMask[(MCAM_SIZE/8)];

#ifdef CONFIG_PROC_FS
    PGETB_PROC_ENTRY            pProcDir;
#endif

    U32                         ticks;
    U32                         rx_bytes;

} GETB_INFO, *PGETB_INFO;

#endif /*__FreeBSD_version*/


typedef struct __get_vendor_info {
    U16         wVendorId;
    U16         wDeviceId;
    U16         wSubVenId;
    U16         wSubDevId;
    PCHIP_INFO  pChip;
} GETB_VENDOR_INFO, *PGETB_VENDOR_INFO;



/*
 * PCI low memory base and low I/O base register, and
 * other PCI registers.
 */
#define GETB_PCI_VENDOR_ID       0x00
#define GETB_PCI_DEVICE_ID       0x02
#define GETB_PCI_COMMAND         0x04
#define GETB_PCI_STATUS          0x06
#define GETB_PCI_REVID           0x08
#define GETB_PCI_CLASSCODE       0x09
#define GETB_PCI_LATENCY_TIMER   0x0D
#define GETB_PCI_HEADER_TYPE     0x0E
#define GETB_PCI_LOIO            0x10
#define GETB_PCI_LOMEM           0x14
#define GETB_PCI_BIOSROM         0x30
#define GETB_PCI_INTLINE         0x3C
#define GETB_PCI_INTPIN          0x3D
#define GETB_PCI_MINGNT          0x3E
#define GETB_PCI_MINLAT          0x0F
#define GETB_PCI_RESETOPT        0x48
#define GETB_PCI_EEPROM_DATA     0x4C
#define GETB_PCI_MODE            0x50

#define PCI_VENDORID(x)     ((x) & 0xFFFF)
#define PCI_CHIPID(x)       (((x) >> 16) & 0xFFFF)

static __inline void getb_update_hw_mibs(PGETB_INFO pInfo) 
{
    U32 dwTmp;
    int i;

    BYTE_REG_BITS_ON(MIBCR_MIBFLSH, &(pInfo->pMacRegs->byMIBCR));

    while (BYTE_REG_BITS_IS_ON(MIBCR_MIBFLSH, &(pInfo->pMacRegs->byMIBCR)));

    BYTE_REG_BITS_ON(MIBCR_MPTRINI, &(pInfo->pMacRegs->byMIBCR));
    for (i=0; i<HW_MIB_SIZE; i++) {
        dwTmp=readl(&(pInfo->pMacRegs->dwMIBData)) & 0x00FFFFFFUL;
        pInfo->adwHWMIBCounters[i]+=dwTmp;
    }
}


static __inline void enable_flow_control(PGETB_INFO pInfo) 
{
    PMAC_REGS   pMacRegs=pInfo->pMacRegs;

    /* Set {XHITH1, XHITH0, XLTH1, XLTH0} in FlowCR1 to {1, 0, 1, 1}
        depend on RD=64, and Turn on XNOEN in FlowCR1*/
    DWORD_REG_BITS_ON((CR0_XONEN|CR0_XHITH1|CR0_XLTH1|CR0_XLTH0),
        &pMacRegs->dwCR0Clr);
    DWORD_REG_BITS_ON((CR0_XONEN|CR0_XHITH1|CR0_XLTH1|CR0_XLTH0),
        &pMacRegs->dwCR0Set);

    /* Set TxPauseTimer to 0xFFFF */
    writew(0xFFFF, &pMacRegs->wTxPauseTimer);

    /* Initialize RBRDU to Rx buffer count.*/
    writeb(pInfo->sOpts.nRxDescs, &pMacRegs->wRBRDU);
}


#endif /* __IF_GETB_H__ */

